@@ -58,8 +58,12 @@ def _backward_hook(
                 ),
                 grad,
             ):
-                surrogate = torch.dot(reps_mb.flatten(), grad_mb.flatten()) * grad_output
-                surrogate.backward()
+                # TODO: This if-statement is for if the model does not require gradients, which may happen if the model
+                # contains a Router where one of the routes is frozen. It should be possible to not have to call
+                # embed_minibatch_iter in that case, as it's unnecessarily expensive.
+                if reps_mb.requires_grad:
+                    surrogate = torch.dot(reps_mb.flatten(), grad_mb.flatten()) * grad_output
+                    surrogate.backward()
 
 
 class CachedGISTEmbedLoss(nn.Module):
@@ -82,7 +86,7 @@ class CachedGISTEmbedLoss(nn.Module):
         :class:`GISTEmbedLoss` yields stronger training signals than :class:`MultipleNegativesRankingLoss` due to the
         use of a guide model for in-batch negative sample selection. Meanwhile, :class:`CachedMultipleNegativesRankingLoss`
         allows for scaling of the batch size by dividing the computation into two stages of embedding and loss
-        calculation, which both can be scaled by mini-batches (https://arxiv.org/pdf/2101.06983.pdf).
+        calculation, which both can be scaled by mini-batches (https://huggingface.co/papers/2101.06983).
 
         By combining the guided selection from :class:`GISTEmbedLoss` and Gradient Cache from
         :class:`CachedMultipleNegativesRankingLoss`, it is possible to reduce memory usage while maintaining performance
@@ -116,9 +120,9 @@ class CachedGISTEmbedLoss(nn.Module):
                 training due to communication overhead, and can potentially lead to out-of-memory errors.
 
         References:
-            - Efficient Natural Language Response Suggestion for Smart Reply, Section 4.4: https://arxiv.org/pdf/1705.00652.pdf
-            - Scaling Deep Contrastive Learning Batch Size under Memory Limited Setup: https://arxiv.org/pdf/2101.06983.pdf
-            - GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning https://arxiv.org/abs/2402.16829
+            - Efficient Natural Language Response Suggestion for Smart Reply, Section 4.4: https://huggingface.co/papers/1705.00652
+            - Scaling Deep Contrastive Learning Batch Size under Memory Limited Setup: https://huggingface.co/papers/2101.06983
+            - GISTEmbed: Guided In-sample Selection of Training Negatives for Text Embedding Fine-tuning https://huggingface.co/papers/2402.16829
 
         Requirements:
             1. (anchor, positive) pairs or (anchor, positive, negative pairs)
@@ -220,7 +224,10 @@ class CachedGISTEmbedLoss(nn.Module):
         """Do forward pass on a minibatch of the input features and return corresponding embeddings."""
         grad_context = nullcontext if with_grad else torch.no_grad
         random_state_context = nullcontext() if random_state is None else random_state
-        sentence_feature_minibatch = {k: v[begin:end] for k, v in sentence_feature.items()}
+        sentence_feature_minibatch = {
+            key: value[begin:end] if isinstance(value, torch.Tensor) else value
+            for key, value in sentence_feature.items()
+        }
         with random_state_context:
             with grad_context():
                 random_state = RandContext(*sentence_feature_minibatch.values()) if copy_random_state else None
